@@ -22,9 +22,13 @@ OQS_KEM *OQS_KEM_classic_mceliece_348864f_new(void) {
 	kem->length_secret_key = OQS_KEM_classic_mceliece_348864f_length_secret_key;
 	kem->length_ciphertext = OQS_KEM_classic_mceliece_348864f_length_ciphertext;
 	kem->length_shared_secret = OQS_KEM_classic_mceliece_348864f_length_shared_secret;
+	kem->length_keypair_seed = OQS_KEM_classic_mceliece_348864f_length_keypair_seed;
+	kem->length_encaps_seed = OQS_KEM_classic_mceliece_348864f_length_encaps_seed;
 
 	kem->keypair = OQS_KEM_classic_mceliece_348864f_keypair;
+	kem->keypair_derand = OQS_KEM_classic_mceliece_348864f_keypair_derand;
 	kem->encaps = OQS_KEM_classic_mceliece_348864f_encaps;
+	kem->encaps_derand = OQS_KEM_classic_mceliece_348864f_encaps_derand;
 	kem->decaps = OQS_KEM_classic_mceliece_348864f_decaps;
 
 	return kem;
@@ -40,6 +44,13 @@ extern int PQCLEAN_MCELIECE348864F_AVX2_crypto_kem_enc(uint8_t *ct, uint8_t *ss,
 extern int PQCLEAN_MCELIECE348864F_AVX2_crypto_kem_dec(uint8_t *ss, const uint8_t *ct, const uint8_t *sk);
 #endif
 
+OQS_API OQS_STATUS OQS_KEM_classic_mceliece_348864f_keypair_derand(uint8_t *public_key, uint8_t *secret_key, const uint8_t *seed) {
+	(void)public_key;
+	(void)secret_key;
+	(void)seed;
+	return OQS_ERROR;
+}
+
 OQS_API OQS_STATUS OQS_KEM_classic_mceliece_348864f_keypair(uint8_t *public_key, uint8_t *secret_key) {
 #if defined(OQS_ENABLE_KEM_classic_mceliece_348864f_avx2)
 #if defined(OQS_DIST_BUILD)
@@ -56,6 +67,14 @@ OQS_API OQS_STATUS OQS_KEM_classic_mceliece_348864f_keypair(uint8_t *public_key,
 #endif
 }
 
+OQS_API OQS_STATUS OQS_KEM_classic_mceliece_348864f_encaps_derand(uint8_t *ciphertext, uint8_t *shared_secret, const uint8_t *public_key, const uint8_t *seed) {
+	(void)ciphertext;
+	(void)shared_secret;
+	(void)public_key;
+	(void)seed;
+	return OQS_ERROR;
+}
+
 OQS_API OQS_STATUS OQS_KEM_classic_mceliece_348864f_encaps(uint8_t *ciphertext, uint8_t *shared_secret, const uint8_t *public_key) {
 #if defined(OQS_ENABLE_KEM_classic_mceliece_348864f_avx2)
 #if defined(OQS_DIST_BUILD)
