@@ -5,6 +5,7 @@
 #define CRYPTO_CIPHERTEXTBYTES OQS_KEM_frodokem_640_aes_length_ciphertext
 #define CRYPTO_BYTES OQS_KEM_frodokem_640_aes_length_shared_secret
 #define CRYPTO_SEEDBYTES OQS_KEM_frodokem_640_aes_length_keypair_seed
+#define CRYPTO_ENCSEEDBYTES OQS_KEM_frodokem_640_aes_length_encaps_seed
 
 // Parameters for "FrodoKEM-640"
 #define PARAMS_N 640
@@ -31,6 +32,7 @@
 #define crypto_kem_keypair            OQS_KEM_frodokem_640_aes_keypair
 #define crypto_kem_keypair_derand     OQS_KEM_frodokem_640_aes_keypair_derand
 #define crypto_kem_enc                OQS_KEM_frodokem_640_aes_encaps
+#define crypto_kem_enc_derand         OQS_KEM_frodokem_640_aes_encaps_derand
 #define crypto_kem_dec                OQS_KEM_frodokem_640_aes_decaps
 
 #define frodo_sample_n oqs_kem_frodokem_640_aes_sample_n
