@@ -22,9 +22,13 @@ OQS_KEM *OQS_KEM_frodokem_1344_aes_new(void) {
 	kem->length_secret_key = OQS_KEM_frodokem_1344_aes_length_secret_key;
 	kem->length_ciphertext = OQS_KEM_frodokem_1344_aes_length_ciphertext;
 	kem->length_shared_secret = OQS_KEM_frodokem_1344_aes_length_shared_secret;
+	kem->length_keypair_seed = OQS_KEM_frodokem_1344_aes_length_keypair_seed;
+	kem->length_encaps_seed = OQS_KEM_frodokem_1344_aes_length_encaps_seed;
 
 	kem->keypair = OQS_KEM_frodokem_1344_aes_keypair;
+	kem->keypair_derand = OQS_KEM_frodokem_1344_aes_keypair_derand;
 	kem->encaps = OQS_KEM_frodokem_1344_aes_encaps;
+	kem->encaps_derand = OQS_KEM_frodokem_1344_aes_encaps_derand;
 	kem->decaps = OQS_KEM_frodokem_1344_aes_decaps;
 
 	return kem;
