@@ -22,9 +22,13 @@ OQS_KEM *OQS_KEM_kyber_1024_new(void) {
 	kem->length_secret_key = OQS_KEM_kyber_1024_length_secret_key;
 	kem->length_ciphertext = OQS_KEM_kyber_1024_length_ciphertext;
 	kem->length_shared_secret = OQS_KEM_kyber_1024_length_shared_secret;
+	kem->length_keypair_seed = OQS_KEM_kyber_1024_length_keypair_seed;
+	kem->length_encaps_seed = OQS_KEM_kyber_1024_length_encaps_seed;
 
 	kem->keypair = OQS_KEM_kyber_1024_keypair;
+	kem->keypair_derand = OQS_KEM_kyber_1024_keypair_derand;
 	kem->encaps = OQS_KEM_kyber_1024_encaps;
+	kem->encaps_derand = OQS_KEM_kyber_1024_encaps_derand;
 	kem->decaps = OQS_KEM_kyber_1024_decaps;
 
 	return kem;
@@ -46,6 +50,13 @@ extern int PQCLEAN_KYBER1024_AARCH64_crypto_kem_enc(uint8_t *ct, uint8_t *ss, co
 extern int PQCLEAN_KYBER1024_AARCH64_crypto_kem_dec(uint8_t *ss, const uint8_t *ct, const uint8_t *sk);
 #endif
 
+OQS_API OQS_STATUS OQS_KEM_kyber_1024_keypair_derand(uint8_t *public_key, uint8_t *secret_key, const uint8_t *seed) {
+	(void)public_key;
+	(void)secret_key;
+	(void)seed;
+	return OQS_ERROR;
+}
+
 OQS_API OQS_STATUS OQS_KEM_kyber_1024_keypair(uint8_t *public_key, uint8_t *secret_key) {
 #if defined(OQS_ENABLE_KEM_kyber_1024_avx2)
 #if defined(OQS_DIST_BUILD)
@@ -72,6 +83,14 @@ OQS_API OQS_STATUS OQS_KEM_kyber_1024_keypair(uint8_t *public_key, uint8_t *secr
 #endif
 }
 
+OQS_API OQS_STATUS OQS_KEM_kyber_1024_encaps_derand(uint8_t *ciphertext, uint8_t *shared_secret, const uint8_t *public_key, const uint8_t *seed) {
+	(void)ciphertext;
+	(void)shared_secret;
+	(void)public_key;
+	(void)seed;
+	return OQS_ERROR;
+}
+
 OQS_API OQS_STATUS OQS_KEM_kyber_1024_encaps(uint8_t *ciphertext, uint8_t *shared_secret, const uint8_t *public_key) {
 #if defined(OQS_ENABLE_KEM_kyber_1024_avx2)
 #if defined(OQS_DIST_BUILD)
