@@ -23,10 +23,12 @@ OQS_KEM *OQS_KEM_kyber_512_new(void) {
 	kem->length_ciphertext = OQS_KEM_kyber_512_length_ciphertext;
 	kem->length_shared_secret = OQS_KEM_kyber_512_length_shared_secret;
 	kem->length_keypair_seed = OQS_KEM_kyber_512_length_keypair_seed;
+	kem->length_encaps_seed = OQS_KEM_kyber_512_length_encaps_seed;
 
 	kem->keypair = OQS_KEM_kyber_512_keypair;
 	kem->keypair_derand = OQS_KEM_kyber_512_keypair_derand;
 	kem->encaps = OQS_KEM_kyber_512_encaps;
+	kem->encaps_derand = OQS_KEM_kyber_512_encaps_derand;
 	kem->decaps = OQS_KEM_kyber_512_decaps;
 
 	return kem;
@@ -110,6 +112,14 @@ OQS_API OQS_STATUS OQS_KEM_kyber_512_keypair(uint8_t *public_key, uint8_t *secre
 #endif /* OQS_LIBJADE_BUILD */
 }
 
+OQS_API OQS_STATUS OQS_KEM_kyber_512_encaps_derand(uint8_t *ciphertext, uint8_t *shared_secret, const uint8_t *public_key, const uint8_t *seed) {
+	(void)ciphertext;
+	(void)shared_secret;
+	(void)public_key;
+	(void)seed;
+	return OQS_ERROR;
+}
+
 OQS_API OQS_STATUS OQS_KEM_kyber_512_encaps(uint8_t *ciphertext, uint8_t *shared_secret, const uint8_t *public_key) {
 #if defined(OQS_LIBJADE_BUILD) && (defined(OQS_ENABLE_LIBJADE_KEM_kyber_512))
 #if defined(OQS_ENABLE_LIBJADE_KEM_kyber_512_avx2)
