@@ -17,16 +17,16 @@
 #include "ds_benchmark.h"
 #include "system_info.c"
 
-static void fullcycle(OQS_SIG *sig, uint8_t *public_key, uint8_t *secret_key, uint8_t *signature, size_t signature_len, uint8_t *message, size_t message_len) {
+static void fullcycle(OQS_SIG *sig, uint8_t *public_key, uint8_t *secret_key, uint8_t *signature, size_t *signature_len, uint8_t *message, size_t message_len) {
 	if (OQS_SIG_keypair(sig, public_key, secret_key) != OQS_SUCCESS) {
 		printf("keygen error. Exiting.\n");
 		exit(-1);
 	}
-	if (OQS_SIG_sign(sig, signature, &signature_len, message, message_len, secret_key) != OQS_SUCCESS) {
+	if (OQS_SIG_sign(sig, signature, signature_len, message, message_len, secret_key) != OQS_SUCCESS) {
 		printf("sign error. Exiting.\n");
 		exit(-1);
 	}
-	if (OQS_SIG_verify(sig, message, message_len, signature, signature_len, public_key) != OQS_SUCCESS) {
+	if (OQS_SIG_verify(sig, message, message_len, signature, *signature_len, public_key) != OQS_SUCCESS) {
 		printf("verify error. Exiting.\n");
 		exit(-1);
 	}
@@ -66,7 +66,7 @@ static OQS_STATUS sig_speed_wrapper(const char *method_name, uint64_t duration,
 		TIME_OPERATION_SECONDS(OQS_SIG_sign(sig, signature, &signature_len, message, message_len, secret_key), "sign", duration)
 		TIME_OPERATION_SECONDS(OQS_SIG_verify(sig, message, message_len, signature, signature_len, public_key), "verify", duration)
 	} else {
-		TIME_OPERATION_SECONDS(fullcycle(sig, public_key, secret_key, signature, signature_len, message, message_len), "fullcycle", duration)
+		TIME_OPERATION_SECONDS(fullcycle(sig, public_key, secret_key, signature, &signature_len, message, message_len), "fullcycle", duration)
 	}
 
 
