@@ -16,29 +16,12 @@
 #include <pthread.h>
 #endif
 
-#ifdef OQS_ENABLE_TEST_CONSTANT_TIME
-#include <valgrind/memcheck.h>
-#define OQS_TEST_CT_CLASSIFY(addr, len)  VALGRIND_MAKE_MEM_UNDEFINED(addr, len)
-#define OQS_TEST_CT_DECLASSIFY(addr, len)  VALGRIND_MAKE_MEM_DEFINED(addr, len)
-#else
-#define OQS_TEST_CT_CLASSIFY(addr, len)
-#define OQS_TEST_CT_DECLASSIFY(addr, len)
-#endif
-
 #ifdef OQS_ENABLE_KEM_ML_KEM
 #define MLKEM_SECRET_LEN      32
 #endif
 
 #include "system_info.c"
-
-/* Displays hexadecimal strings */
-static void OQS_print_hex_string(const char *label, const uint8_t *str, size_t len) {
-	printf("%-20s (%4zu bytes):  ", label, len);
-	for (size_t i = 0; i < (len); i++) {
-		printf("%02X", str[i]);
-	}
-	printf("\n");
-}
+#include "test_helpers.h"
 
 #ifdef OQS_ENABLE_KEM_ML_KEM
 /* mlkem rejection key testcase */
@@ -89,7 +72,7 @@ static bool mlkem_rej_testcase(OQS_KEM *kem, uint8_t *ciphertext, uint8_t *secre
 		goto cleanup;
 	}
 	secret_key[0] -= 1; // Restore private key
-	memset(buff_z_c, 0, length_z_c); // Reset buffer
+	OQS_MEM_cleanse(buff_z_c, length_z_c); // Reset buffer
 
 	// Scenario 2: Test rejection key by corrupting the ciphertext
 	ciphertext[0] += 1;
@@ -127,7 +110,7 @@ typedef struct magic_s {
 	uint8_t val[31];
 } magic_t;
 
-static OQS_STATUS kem_test_correctness(const char *method_name) {
+static OQS_STATUS kem_test_correctness(const char *method_name, bool derand) {
 
 	OQS_KEM *kem = NULL;
 	uint8_t *public_key = NULL;
@@ -135,6 +118,8 @@ static OQS_STATUS kem_test_correctness(const char *method_name) {
 	uint8_t *ciphertext = NULL;
 	uint8_t *shared_secret_e = NULL;
 	uint8_t *shared_secret_d = NULL;
+	uint8_t *keypair_seed = NULL;
+	uint8_t *encaps_seed = NULL;
 	OQS_STATUS rc, ret = OQS_ERROR;
 	int rv;
 
@@ -149,54 +134,128 @@ static OQS_STATUS kem_test_correctness(const char *method_name) {
 		goto err;
 	}
 
-	printf("================================================================================\n");
-	printf("Sample computation for KEM %s\n", kem->method_name);
-	printf("================================================================================\n");
+	if (!derand) {
+		printf("================================================================================\n");
+		printf("sample computation for KEM %s\n", kem->method_name);
+		printf("Version source: %s\n", kem->alg_version);
+		printf("================================================================================\n");
+	}
 
+	/* pk, sk, ciphertext, shared secret, and shared secret cmp are needed
+	 * regardless of derand or not
+	 */
 	public_key = OQS_MEM_malloc(kem->length_public_key + 2 * sizeof(magic_t));
 	secret_key = OQS_MEM_malloc(kem->length_secret_key + 2 * sizeof(magic_t));
 	ciphertext = OQS_MEM_malloc(kem->length_ciphertext + 2 * sizeof(magic_t));
 	shared_secret_e = OQS_MEM_malloc(kem->length_shared_secret + 2 * sizeof(magic_t));
 	shared_secret_d = OQS_MEM_malloc(kem->length_shared_secret + 2 * sizeof(magic_t));
-
 	if ((public_key == NULL) || (secret_key == NULL) || (ciphertext == NULL) || (shared_secret_e == NULL) || (shared_secret_d == NULL)) {
 		fprintf(stderr, "ERROR: OQS_MEM_malloc failed\n");
 		goto err;
 	}
-
-	//Set the magic numbers before
 	memcpy(public_key, magic.val, sizeof(magic_t));
 	memcpy(secret_key, magic.val, sizeof(magic_t));
 	memcpy(ciphertext, magic.val, sizeof(magic_t));
 	memcpy(shared_secret_e, magic.val, sizeof(magic_t));
 	memcpy(shared_secret_d, magic.val, sizeof(magic_t));
-
 	public_key += sizeof(magic_t);
 	secret_key += sizeof(magic_t);
 	ciphertext += sizeof(magic_t);
 	shared_secret_e += sizeof(magic_t);
 	shared_secret_d += sizeof(magic_t);
-
-	// and after
 	memcpy(public_key + kem->length_public_key, magic.val, sizeof(magic_t));
 	memcpy(secret_key + kem->length_secret_key, magic.val, sizeof(magic_t));
 	memcpy(ciphertext + kem->length_ciphertext, magic.val, sizeof(magic_t));
 	memcpy(shared_secret_e + kem->length_shared_secret, magic.val, sizeof(magic_t));
 	memcpy(shared_secret_d + kem->length_shared_secret, magic.val, sizeof(magic_t));
 
-	rc = OQS_KEM_keypair(kem, public_key, secret_key);
-	OQS_TEST_CT_DECLASSIFY(&rc, sizeof rc);
-	if (rc != OQS_SUCCESS) {
-		fprintf(stderr, "ERROR: OQS_KEM_keypair failed\n");
-		goto err;
+	if (derand) {
+		/* keypair_seed and encaps_seed are only needed if derand */
+		keypair_seed = malloc(kem->length_keypair_seed + 2 * sizeof(magic_t));
+		if (!keypair_seed) {
+			fprintf(stderr, "Failed to allocate %zu bytes for keypair seed\n",
+			        kem->length_keypair_seed);
+			goto err;
+		}
+		encaps_seed = malloc(kem->length_encaps_seed + 2 * sizeof(magic_t));
+		if (!encaps_seed) {
+			fprintf(stderr, "Failed to allocate %zu bytes for encaps seed\n",
+			        kem->length_encaps_seed);
+			goto err;
+		}
+		memcpy(keypair_seed, magic.val, sizeof(magic_t));
+		memcpy(encaps_seed, magic.val, sizeof(magic_t));
+		keypair_seed += sizeof(magic_t);
+		encaps_seed += sizeof(magic_t);
+		memcpy(keypair_seed + kem->length_keypair_seed, magic.val,
+		       sizeof(magic_t));
+		memcpy(encaps_seed + kem->length_encaps_seed, magic.val,
+		       sizeof(magic_t));
+	}
+
+
+
+	if (derand) {
+		// On some systems, getentropy fails if given a zero-length array
+		if (kem->length_keypair_seed > 0) {
+			OQS_randombytes(keypair_seed, kem->length_keypair_seed);
+		}
+		rc = OQS_KEM_keypair_derand(kem, public_key, secret_key, keypair_seed);
+		OQS_TEST_CT_DECLASSIFY(&rc, sizeof rc);
+		if (kem->length_keypair_seed == 0) {
+			// If length_keypair_seed is set to 0 for this KEM scheme, a failure is expected
+			if (rc != OQS_ERROR) {
+				fprintf(stderr, "ERROR: OQS_KEM_keypair_derand succeeded but expected a failure\n");
+				goto err;
+			}
+			printf("OQS_KEM_keypair_derand failed, as expected\n");
+			ret = OQS_SUCCESS;
+			goto cleanup;
+		} else {
+			if (rc != OQS_SUCCESS) {
+				fprintf(stderr, "ERROR: OQS_KEM_keypair_derand failed\n");
+				goto err;
+			}
+		}
+	} else {
+		rc = OQS_KEM_keypair(kem, public_key, secret_key);
+		OQS_TEST_CT_DECLASSIFY(&rc, sizeof rc);
+		if (rc != OQS_SUCCESS) {
+			fprintf(stderr, "ERROR: OQS_KEM_keypair failed\n");
+			goto err;
+		}
 	}
 
 	OQS_TEST_CT_DECLASSIFY(public_key, kem->length_public_key);
-	rc = OQS_KEM_encaps(kem, ciphertext, shared_secret_e, public_key);
-	OQS_TEST_CT_DECLASSIFY(&rc, sizeof rc);
-	if (rc != OQS_SUCCESS) {
-		fprintf(stderr, "ERROR: OQS_KEM_encaps failed\n");
-		goto err;
+	if (derand) {
+		// On some systems, getentropy fails if given a zero-length array
+		if (kem->length_encaps_seed > 0) {
+			OQS_randombytes(encaps_seed, kem->length_encaps_seed);
+		}
+		rc = OQS_KEM_encaps_derand(kem, ciphertext, shared_secret_e, public_key, encaps_seed);
+		OQS_TEST_CT_DECLASSIFY(&rc, sizeof rc);
+		if (kem->length_encaps_seed == 0) {
+			// If length_encaps_seed is set to 0 for this KEM scheme, a failure is expected
+			if (rc != OQS_ERROR) {
+				fprintf(stderr, "ERROR: OQS_KEM_encaps_derand succeeded but expected a failure\n");
+				goto err;
+			}
+			printf("OQS_KEM_encaps_derand failed, as expected\n");
+			ret = OQS_SUCCESS;
+			goto cleanup;
+		} else {
+			if (rc != OQS_SUCCESS) {
+				fprintf(stderr, "ERROR: OQS_KEM_encaps_derand failed\n");
+				goto err;
+			}
+		}
+	} else {
+		rc = OQS_KEM_encaps(kem, ciphertext, shared_secret_e, public_key);
+		OQS_TEST_CT_DECLASSIFY(&rc, sizeof rc);
+		if (rc != OQS_SUCCESS) {
+			fprintf(stderr, "ERROR: OQS_KEM_encaps failed\n");
+			goto err;
+		}
 	}
 
 	OQS_TEST_CT_DECLASSIFY(ciphertext, kem->length_ciphertext);
@@ -248,6 +307,12 @@ static OQS_STATUS kem_test_correctness(const char *method_name) {
 	rv |= memcmp(ciphertext - sizeof(magic_t), magic.val, sizeof(magic_t));
 	rv |= memcmp(shared_secret_e - sizeof(magic_t), magic.val, sizeof(magic_t));
 	rv |= memcmp(shared_secret_d - sizeof(magic_t), magic.val, sizeof(magic_t));
+	if (derand) {
+		rv |= memcmp(keypair_seed + kem->length_keypair_seed, magic.val, sizeof(magic_t));
+		rv |= memcmp(encaps_seed + kem->length_encaps_seed, magic.val, sizeof(magic_t));
+		rv |= memcmp(keypair_seed - sizeof(magic_t), magic.val, sizeof(magic_t));
+		rv |= memcmp(encaps_seed - sizeof(magic_t), magic.val, sizeof(magic_t));
+	}
 	if (rv != 0) {
 		fprintf(stderr, "ERROR: Magic numbers do not match\n");
 		goto err;
@@ -261,13 +326,13 @@ err:
 	ret = OQS_ERROR;
 
 cleanup:
-	if (secret_key) {
+	if ((secret_key) && (kem != NULL)) {
 		OQS_MEM_secure_free(secret_key - sizeof(magic_t), kem->length_secret_key + 2 * sizeof(magic_t));
 	}
-	if (shared_secret_e) {
+	if ((shared_secret_e) && (kem != NULL)) {
 		OQS_MEM_secure_free(shared_secret_e - sizeof(magic_t), kem->length_shared_secret + 2 * sizeof(magic_t));
 	}
-	if (shared_secret_d) {
+	if ((shared_secret_d) && (kem != NULL)) {
 		OQS_MEM_secure_free(shared_secret_d - sizeof(magic_t), kem->length_shared_secret + 2 * sizeof(magic_t));
 	}
 	if (public_key) {
@@ -276,6 +341,12 @@ cleanup:
 	if (ciphertext) {
 		OQS_MEM_insecure_free(ciphertext - sizeof(magic_t));
 	}
+	if ((keypair_seed) && (kem != NULL)) {
+		OQS_MEM_secure_free(keypair_seed - sizeof(magic_t), kem->length_keypair_seed + 2 * sizeof(magic_t));
+	}
+	if ((encaps_seed) && (kem != NULL)) {
+		OQS_MEM_secure_free(encaps_seed - sizeof(magic_t), kem->length_encaps_seed + 2 * sizeof(magic_t));
+	}
 	OQS_KEM_free(kem);
 
 	return ret;
@@ -285,7 +356,7 @@ cleanup:
 static void TEST_KEM_randombytes(uint8_t *random_array, size_t bytes_to_read) {
 	// We can't make direct calls to the system randombytes on some platforms,
 	// so we have to swap out the OQS_randombytes provider.
-	OQS_randombytes_switch_algorithm("system");
+	(void)OQS_randombytes_switch_algorithm("system");
 	OQS_randombytes(random_array, bytes_to_read);
 	OQS_randombytes_custom_algorithm(&TEST_KEM_randombytes);
 
@@ -304,13 +375,18 @@ struct thread_data {
 
 void *test_wrapper(void *arg) {
 	struct thread_data *td = arg;
-	td->rc = kem_test_correctness(td->alg_name);
+	td->rc = kem_test_correctness(td->alg_name, false);
+	if (td->rc == OQS_SUCCESS) {
+		// test derandomized operations
+		td->rc = kem_test_correctness(td->alg_name, true);
+	}
 	OQS_thread_stop();
 	return NULL;
 }
 #endif
 
 int main(int argc, char **argv) {
+	OQS_STATUS rc;
 	OQS_init();
 
 	printf("Testing KEM algorithms using liboqs version %s\n", OQS_version());
@@ -341,10 +417,14 @@ int main(int argc, char **argv) {
 #ifdef OQS_ENABLE_TEST_CONSTANT_TIME
 	OQS_randombytes_custom_algorithm(&TEST_KEM_randombytes);
 #else
-	OQS_randombytes_switch_algorithm("system");
+	rc = OQS_randombytes_switch_algorithm("system");
+	if (rc != OQS_SUCCESS) {
+		printf("Could not generate random data with system RNG\n");
+		OQS_destroy();
+		return EXIT_FAILURE;
+	}
 #endif
 
-	OQS_STATUS rc;
 #if OQS_USE_PTHREADS
 #define MAX_LEN_KEM_NAME_ 64
 	// don't run Classic McEliece in threads because of large stack usage
@@ -369,10 +449,18 @@ int main(int argc, char **argv) {
 		pthread_join(thread, NULL);
 		rc = td.rc;
 	} else {
-		rc = kem_test_correctness(alg_name);
+		rc = kem_test_correctness(alg_name, false);
+		if (rc == OQS_SUCCESS) {
+			// test with derandomized keygen
+			rc = kem_test_correctness(alg_name, true);
+		}
 	}
 #else
-	rc = kem_test_correctness(alg_name);
+	rc = kem_test_correctness(alg_name, false);
+	if (rc == OQS_SUCCESS) {
+		// test with derandomized keygen
+		rc = kem_test_correctness(alg_name, true);
+	}
 #endif
 	if (rc != OQS_SUCCESS) {
 		OQS_destroy();
