@@ -21,7 +21,7 @@ typedef struct magic_s {
 	uint8_t val[31];
 } magic_t;
 
-static OQS_STATUS sig_test_correctness(const char *method_name, bool bitflips_all[2], size_t bitflips[2]) {
+static OQS_STATUS sig_test_correctness(const char *method_name, bool bitflips_all[2], size_t bitflips[2], bool extended_tests) {
 
 	OQS_SIG *sig = NULL;
 	uint8_t *public_key = NULL;
@@ -38,6 +38,9 @@ static OQS_STATUS sig_test_correctness(const char *method_name, bool bitflips_al
 	magic_t magic;
 	OQS_randombytes(magic.val, sizeof(magic_t));
 
+	printf(extended_tests ? "Extended tests enabled\n" : "Extended tests disabled\n");
+
+
 	sig = OQS_SIG_new(method_name);
 	if (sig == NULL) {
 		fprintf(stderr, "ERROR: OQS_SIG_new failed\n");
@@ -49,7 +52,7 @@ static OQS_STATUS sig_test_correctness(const char *method_name, bool bitflips_al
 		if (bitflips_all[i]) {
 			snprintf(bitflips_as_str[i], sizeof(bitflips_as_str[i]), "all");
 		} else {
-			snprintf(bitflips_as_str[i], sizeof(bitflips_as_str[i]), "%ld random", bitflips[i]);
+			snprintf(bitflips_as_str[i], sizeof(bitflips_as_str[i]), "%zu random", bitflips[i]);
 		}
 	}
 
@@ -117,48 +120,59 @@ static OQS_STATUS sig_test_correctness(const char *method_name, bool bitflips_al
 		goto err;
 	}
 
-	rc = test_sig_bitflip(sig, message, message_len, signature, signature_len, public_key, bitflips_all, bitflips, false, NULL, 0);
-	OQS_TEST_CT_DECLASSIFY(&rc, sizeof rc);
-	if (rc != OQS_SUCCESS) {
-		goto err;
+	if (extended_tests) {
+		rc = test_sig_bitflip(sig, message, message_len, signature, signature_len, public_key, bitflips_all, bitflips, false, NULL, 0);
+		OQS_TEST_CT_DECLASSIFY(&rc, sizeof rc);
+		if (rc != OQS_SUCCESS) {
+			goto err;
+		}
 	}
 
-
 	/* testing signing with context, if supported */
 	OQS_randombytes(ctx, 257);
 	if (sig->sig_with_ctx_support) {
+		size_t ctx_step = 1;
+		// Only do a small fraction of the context sizes for SLH_DSA for efficiency purposes
+		if (!strncmp(sig->method_name, "SLH_DSA", 7)) {
+			ctx_step = 61; // using a prime slightly smaller than a power of 2 to avoid only testing word/block aligned values
+		}
 		for (size_t i = 0; i < 256; ++i) {
-			rc = OQS_SIG_sign_with_ctx_str(sig, signature, &signature_len, message, message_len, ctx, i, secret_key);
-			OQS_TEST_CT_DECLASSIFY(&rc, sizeof rc);
-			if (rc != OQS_SUCCESS) {
-				fprintf(stderr, "ERROR: OQS_SIG_sign_with_ctx_str failed\n");
-				goto err;
-			}
+			if (((i % ctx_step == 0) && extended_tests) || i == 255) {
+				rc = OQS_SIG_sign_with_ctx_str(sig, signature, &signature_len, message, message_len, ctx, i, secret_key);
+				OQS_TEST_CT_DECLASSIFY(&rc, sizeof rc);
+				if (rc != OQS_SUCCESS) {
+					fprintf(stderr, "ERROR: OQS_SIG_sign_with_ctx_str failed\n");
+					goto err;
+				}
 
-			OQS_TEST_CT_DECLASSIFY(public_key, sig->length_public_key);
-			OQS_TEST_CT_DECLASSIFY(signature, signature_len);
-			rc = OQS_SIG_verify_with_ctx_str(sig, message, message_len, signature, signature_len, ctx, i, public_key);
-			OQS_TEST_CT_DECLASSIFY(&rc, sizeof rc);
-			if (rc != OQS_SUCCESS) {
-				fprintf(stderr, "ERROR: OQS_SIG_verify_with_ctx_str failed\n");
-				goto err;
-			}
+				OQS_TEST_CT_DECLASSIFY(public_key, sig->length_public_key);
+				OQS_TEST_CT_DECLASSIFY(signature, signature_len);
+				rc = OQS_SIG_verify_with_ctx_str(sig, message, message_len, signature, signature_len, ctx, i, public_key);
+				OQS_TEST_CT_DECLASSIFY(&rc, sizeof rc);
+				if (rc != OQS_SUCCESS) {
+					fprintf(stderr, "ERROR: OQS_SIG_verify_with_ctx_str failed\n");
+					goto err;
+				}
 
-			rc = test_sig_bitflip(sig, message, message_len, signature, signature_len, public_key, bitflips_all, bitflips, true, ctx, i);
-			OQS_TEST_CT_DECLASSIFY(&rc, sizeof rc);
-			if (rc != OQS_SUCCESS) {
-				goto err;
+				if (extended_tests) {
+					rc = test_sig_bitflip(sig, message, message_len, signature, signature_len, public_key, bitflips_all, bitflips, true, ctx, i);
+					OQS_TEST_CT_DECLASSIFY(&rc, sizeof rc);
+					if (rc != OQS_SUCCESS) {
+						goto err;
+					}
+				}
 			}
-
 		}
 
-		rc = OQS_SIG_sign_with_ctx_str(sig, signature, &signature_len, message, message_len, ctx, 256, secret_key);
-		OQS_TEST_CT_DECLASSIFY(&rc, sizeof rc);
-		if (rc != OQS_ERROR) {
-			fprintf(stderr, "ERROR: OQS_SIG_sign_with_ctx_str should only support up to 255 byte contexts\n");
-			goto err;
+		if (extended_tests) {
+			rc = OQS_SIG_sign_with_ctx_str(sig, signature, &signature_len, message, message_len, ctx, 256, secret_key);
+			OQS_TEST_CT_DECLASSIFY(&rc, sizeof rc);
+			if (rc != OQS_ERROR) {
+				fprintf(stderr, "ERROR: OQS_SIG_sign_with_ctx_str should only support up to 255 byte contexts\n");
+				goto err;
+			}
 		}
-	} else {
+	} else if (extended_tests) {
 		rc = OQS_SIG_sign_with_ctx_str(sig, signature, &signature_len, message, message_len, ctx, 1, secret_key);
 		if (rc != OQS_ERROR) {
 			fprintf(stderr, "ERROR: OQS_SIG_sign_with_ctx_str should fail without support for context strings\n");
@@ -166,25 +180,27 @@ static OQS_STATUS sig_test_correctness(const char *method_name, bool bitflips_al
 		}
 	}
 
-	rc = OQS_SIG_sign_with_ctx_str(sig, signature, &signature_len, message, message_len, NULL, 0, secret_key);
-	OQS_TEST_CT_DECLASSIFY(&rc, sizeof rc);
-	if (rc != OQS_SUCCESS) {
-		fprintf(stderr, "ERROR: OQS_SIG_sign_with_ctx_str should always succeed when providing a NULL context string\n");
-		goto err;
-	}
-	OQS_TEST_CT_DECLASSIFY(public_key, sig->length_public_key);
-	OQS_TEST_CT_DECLASSIFY(signature, signature_len);
-	rc = OQS_SIG_verify_with_ctx_str(sig, message, message_len, signature, signature_len, NULL, 0, public_key);
-	OQS_TEST_CT_DECLASSIFY(&rc, sizeof rc);
-	if (rc != OQS_SUCCESS) {
-		fprintf(stderr, "ERROR: OQS_SIG_verify_with_ctx_str failed\n");
-		goto err;
-	}
+	if (extended_tests) {
+		rc = OQS_SIG_sign_with_ctx_str(sig, signature, &signature_len, message, message_len, NULL, 0, secret_key);
+		OQS_TEST_CT_DECLASSIFY(&rc, sizeof rc);
+		if (rc != OQS_SUCCESS) {
+			fprintf(stderr, "ERROR: OQS_SIG_sign_with_ctx_str should always succeed when providing a NULL context string\n");
+			goto err;
+		}
+		OQS_TEST_CT_DECLASSIFY(public_key, sig->length_public_key);
+		OQS_TEST_CT_DECLASSIFY(signature, signature_len);
+		rc = OQS_SIG_verify_with_ctx_str(sig, message, message_len, signature, signature_len, NULL, 0, public_key);
+		OQS_TEST_CT_DECLASSIFY(&rc, sizeof rc);
+		if (rc != OQS_SUCCESS) {
+			fprintf(stderr, "ERROR: OQS_SIG_verify_with_ctx_str failed\n");
+			goto err;
+		}
 
-	rc = test_sig_bitflip(sig, message, message_len, signature, signature_len, public_key, bitflips_all, bitflips, true, NULL, 0);
-	OQS_TEST_CT_DECLASSIFY(&rc, sizeof rc);
-	if (rc != OQS_SUCCESS) {
-		goto err;
+		rc = test_sig_bitflip(sig, message, message_len, signature, signature_len, public_key, bitflips_all, bitflips, true, NULL, 0);
+		OQS_TEST_CT_DECLASSIFY(&rc, sizeof rc);
+		if (rc != OQS_SUCCESS) {
+			goto err;
+		}
 	}
 
 #ifndef OQS_ENABLE_TEST_CONSTANT_TIME
@@ -248,12 +264,13 @@ struct thread_data {
 	char *alg_name;
 	bool *bitflips_all;
 	size_t *bitflips;
+	bool extended_tests;
 	OQS_STATUS rc;
 };
 
 void *test_wrapper(void *arg) {
 	struct thread_data *td = arg;
-	td->rc = sig_test_correctness(td->alg_name, td->bitflips_all, td->bitflips);
+	td->rc = sig_test_correctness(td->alg_name, td->bitflips_all, td->bitflips, td->extended_tests);
 	OQS_thread_stop();
 	return NULL;
 }
@@ -262,11 +279,15 @@ void *test_wrapper(void *arg) {
 int main(int argc, char **argv) {
 	OQS_STATUS rc;
 	OQS_init();
-
+#if defined(OQS_ENABLE_TEST_CONSTANT_TIME) || defined(USE_SANITIZER)
+	long int extended_tests = 0;
+#else
+	long int extended_tests = 1;
+#endif
 	printf("Testing signature algorithms using liboqs version %s\n", OQS_version());
 
-	if (argc < 2 || argc > 4) {
-		fprintf(stderr, "Usage: test_sig algname [bitflips_msg] [bitflips_sig]\n");
+	if (argc < 2 || argc > 5) {
+		fprintf(stderr, "Usage: test_sig algname [bitflips_msg] [bitflips_sig] [extended_tests]\n");
 		fprintf(stderr, "  algname: ");
 		for (size_t i = 0; i < OQS_SIG_algs_length; i++) {
 			if (i > 0) {
@@ -275,6 +296,7 @@ int main(int argc, char **argv) {
 			fprintf(stderr, "%s", OQS_SIG_alg_identifier(i));
 		}
 		fprintf(stderr, "\n");
+		fprintf(stderr, "  extended_tests: run extended correctness tests (with bitflips, full context-string tests)\n");
 		fprintf(stderr, "  bitflips_msg: the number of random bitflips to perform for each EUF-CMA signature (\"all\" to flip every bit)\n");
 		fprintf(stderr, "  bitflips_sig: the number of random bitflips to perform for each SUF-CMA signature (\"all\" to flip every bit)\n");
 		OQS_destroy();
@@ -300,13 +322,21 @@ int main(int argc, char **argv) {
 			bitflips[0] = (size_t)strtol(argv[2], NULL, 10);
 		}
 	}
-	if (argc == 4) {
+	if (argc >= 4) {
 		if (strcmp(argv[3], "all") == 0) {
 			bitflips_all[1] = true;
 		} else {
 			bitflips[1] = (size_t)strtol(argv[3], NULL, 10);
 		}
 	}
+	if (argc == 5) {
+		extended_tests = strtol(argv[4], NULL, 10);
+		if (extended_tests != 0 && extended_tests != 1) {
+			fprintf(stderr, "ERROR: invalid value for extended_tests (must be 0 or 1)\n");
+			OQS_destroy();
+			return EXIT_FAILURE;
+		}
+	}
 
 #ifdef OQS_ENABLE_TEST_CONSTANT_TIME
 	OQS_randombytes_custom_algorithm(&TEST_SIG_randombytes);
@@ -319,20 +349,20 @@ int main(int argc, char **argv) {
 	}
 #endif
 
-#if OQS_USE_PTHREADS
+#if OQS_USE_PTHREADS && !defined(OQS_ENABLE_TEST_CONSTANT_TIME)
 #define MAX_LEN_SIG_NAME_ 64
 	// don't run algorithms with large stack usage in threads
 	char no_thread_sig_patterns[][MAX_LEN_SIG_NAME_]  = {"MAYO-5", "cross-rsdp-128-small", "cross-rsdp-192-small", "cross-rsdp-256-balanced", "cross-rsdp-256-small", "cross-rsdpg-192-small", "cross-rsdpg-256-small", "SNOVA_37_17_2", "SNOVA_56_25_2", "SNOVA_49_11_3", "SNOVA_37_8_4", "SNOVA_24_5_5", "SNOVA_60_10_4", "SNOVA_29_6_5"};
 	int test_in_thread = 1;
 	for (size_t i = 0 ; i < sizeof(no_thread_sig_patterns) / MAX_LEN_SIG_NAME_; ++i) {
-		if (strstr(alg_name, no_thread_sig_patterns[i]) != NULL) {
+		if ( (strncmp(alg_name, "SLH_DSA", 7) == 0) || (strstr(alg_name, no_thread_sig_patterns[i]) != NULL) ) {
 			test_in_thread = 0;
 			break;
 		}
 	}
 	if (test_in_thread) {
 		pthread_t thread;
-		struct thread_data td = {.alg_name = alg_name, .bitflips_all = bitflips_all, .bitflips = bitflips, .rc = OQS_ERROR};
+		struct thread_data td = {.alg_name = alg_name, .bitflips_all = bitflips_all, .bitflips = bitflips, .rc = OQS_ERROR, .extended_tests = (bool)extended_tests};
 		int trc = pthread_create(&thread, NULL, test_wrapper, &td);
 		if (trc) {
 			fprintf(stderr, "ERROR: Creating pthread\n");
@@ -342,10 +372,10 @@ int main(int argc, char **argv) {
 		pthread_join(thread, NULL);
 		rc = td.rc;
 	} else {
-		rc = sig_test_correctness(alg_name, bitflips_all, bitflips);
+		rc = sig_test_correctness(alg_name, bitflips_all, bitflips, (bool)extended_tests);
 	}
 #else
-	rc = sig_test_correctness(alg_name, bitflips_all, bitflips);
+	rc = sig_test_correctness(alg_name, bitflips_all, bitflips, (bool)extended_tests);
 #endif
 	if (rc != OQS_SUCCESS) {
 		OQS_destroy();
