@@ -19,23 +19,14 @@
 #include <oqs/oqs.h>
 #include "tmp_store.c"
 #include "system_info.c"
+#include "test_helpers.h"
 
 #if OQS_USE_PTHREADS_IN_TESTS
 #include <pthread.h>
-
 static pthread_mutex_t *test_sk_lock = NULL;
 static pthread_mutex_t *sk_lock = NULL;
 #endif
 
-#ifdef OQS_ENABLE_TEST_CONSTANT_TIME
-#include <valgrind/memcheck.h>
-#define OQS_TEST_CT_CLASSIFY(addr, len)  VALGRIND_MAKE_MEM_UNDEFINED(addr, len)
-#define OQS_TEST_CT_DECLASSIFY(addr, len)  VALGRIND_MAKE_MEM_DEFINED(addr, len)
-#else
-#define OQS_TEST_CT_CLASSIFY(addr, len)
-#define OQS_TEST_CT_DECLASSIFY(addr, len)
-#endif
-
 #ifdef __GNUC__
 #define UNUSED __attribute__((unused))
 #else
@@ -103,7 +94,7 @@ int ReadHex(FILE *infile, unsigned char *a, unsigned long Length, char *str) {
 		a[0] = 0x00;
 		return 1;
 	}
-	memset(a, 0x00, Length);
+	OQS_MEM_cleanse(a, Length);
 	started = 0;
 	if (FindMarker(infile, str))
 		while ((ch = fgetc(infile)) != EOF) {
@@ -408,7 +399,7 @@ static char *convert_method_name_to_file_name(const char *method_name) {
 	return strdup(file_store);
 }
 
-static OQS_STATUS sig_stfl_test_correctness(const char *method_name, const char *katfile) {
+static OQS_STATUS sig_stfl_test_correctness(const char *method_name, const char *katfile, bool bitflips_all[2], size_t bitflips[2]) {
 
 	OQS_SIG_STFL *sig = NULL;
 	uint8_t *public_key = NULL;
@@ -440,8 +431,25 @@ static OQS_STATUS sig_stfl_test_correctness(const char *method_name, const char
 		goto err;
 	}
 
+	char bitflips_as_str[2][50];
+	for (int i = 0; i < 2; i++) {
+		if (bitflips_all[i]) {
+			snprintf(bitflips_as_str[i], sizeof(bitflips_as_str[i]), "all");
+		} else {
+			snprintf(bitflips_as_str[i], sizeof(bitflips_as_str[i]), "%zu random", bitflips[i]);
+		}
+	}
+
 	printf("================================================================================\n");
 	printf("Sample computation for stateful signature %s\n", sig->method_name);
+	printf("Sample computation for signature %s\n", sig->method_name);
+	if (sig->euf_cma) {
+		printf("Testing EUF-CMA by flipping %s bits of the message\n", bitflips_as_str[0]);
+	}
+	if (sig->suf_cma) {
+		printf("Testing SUF-CMA by flipping %s bits of the signature\n", bitflips_as_str[1]);
+	}
+	printf("Version source: %s\n", sig->alg_version);
 	printf("================================================================================\n");
 
 	secret_key = OQS_SIG_STFL_SECRET_KEY_new(sig->method_name);
@@ -463,12 +471,12 @@ static OQS_STATUS sig_stfl_test_correctness(const char *method_name, const char
 #if OQS_USE_PTHREADS_IN_TESTS
 	OQS_SIG_STFL_SECRET_KEY_SET_mutex(secret_key, sk_lock);
 #endif
-	public_key = malloc(sig->length_public_key + 2 * sizeof(magic_t));
-	message = malloc(message_len + 2 * sizeof(magic_t));
-	signature = malloc(sig->length_signature + 2 * sizeof(magic_t));
+	public_key = OQS_MEM_malloc(sig->length_public_key + 2 * sizeof(magic_t));
+	message = OQS_MEM_malloc(message_len + 2 * sizeof(magic_t));
+	signature = OQS_MEM_malloc(sig->length_signature + 2 * sizeof(magic_t));
 
 	if ((public_key == NULL) || (secret_key == NULL) || (message == NULL) || (signature == NULL)) {
-		fprintf(stderr, "ERROR: malloc failed\n");
+		fprintf(stderr, "ERROR: OQS_MEM_malloc failed\n");
 		goto err;
 	}
 
@@ -530,7 +538,7 @@ static OQS_STATUS sig_stfl_test_correctness(const char *method_name, const char
 	}
 
 	/* Read public key and re-test verify.*/
-	read_pk_buf = malloc(sig->length_public_key);
+	read_pk_buf = OQS_MEM_malloc(sig->length_public_key);
 	if (oqs_fload("pk", file_store, read_pk_buf, sig->length_public_key, &read_pk_len) != OQS_SUCCESS) {
 		goto err;
 	}
@@ -540,13 +548,9 @@ static OQS_STATUS sig_stfl_test_correctness(const char *method_name, const char
 		fprintf(stderr, "ERROR: 2nd Verify with restored public key OQS_SIG_STFL_verify failed\n");
 	}
 
-	/* modify the signature to invalidate it */
-	OQS_randombytes(signature, signature_len);
-	OQS_TEST_CT_DECLASSIFY(signature, signature_len);
-	rc = OQS_SIG_STFL_verify(sig, message, message_len, signature, signature_len, public_key);
+	rc = test_sig_stfl_bitflip(sig, message, message_len, signature, signature_len, public_key, bitflips_all, bitflips);
 	OQS_TEST_CT_DECLASSIFY(&rc, sizeof rc);
-	if (rc != OQS_ERROR) {
-		fprintf(stderr, "ERROR: OQS_SIG_STFL_verify should have failed!\n");
+	if (rc != OQS_SUCCESS) {
 		goto err;
 	}
 
@@ -621,7 +625,7 @@ static OQS_STATUS sig_stfl_test_secret_key(const char *method_name, const char *
 		goto err;
 	}
 
-	public_key = malloc(sig_obj->length_public_key * sizeof(uint8_t));
+	public_key = OQS_MEM_malloc(sig_obj->length_public_key * sizeof(uint8_t));
 
 	printf("================================================================================\n");
 	printf("Create stateful Secret Key  %s\n", method_name);
@@ -682,7 +686,7 @@ static OQS_STATUS sig_stfl_test_secret_key(const char *method_name, const char *
 	}
 
 	/* read secret key from disk */
-	from_file_sk_buf = malloc(to_file_sk_len);
+	from_file_sk_buf = OQS_MEM_malloc(to_file_sk_len);
 	if (oqs_fload("sk", file_store_name, from_file_sk_buf, to_file_sk_len, &from_file_sk_len) != OQS_SUCCESS) {
 		goto err;
 	}
@@ -730,7 +734,7 @@ static void TEST_SIG_STFL_randombytes(uint8_t *random_array, size_t bytes_to_rea
 	// We can't make direct calls to the system randombytes on some platforms,
 	// so we have to swap out the OQS_randombytes provider.
 
-	OQS_randombytes_switch_algorithm("system");
+	(void)OQS_randombytes_switch_algorithm("system");
 	OQS_randombytes(random_array, bytes_to_read);
 	OQS_randombytes_custom_algorithm(&TEST_SIG_STFL_randombytes);
 
@@ -837,7 +841,7 @@ static OQS_STATUS sig_stfl_test_sig_gen(const char *method_name) {
 	printf("Sig Gen 1  %s\n", method_name);
 	printf("================================================================================\n");
 
-	signature_1 = malloc(lock_test_sig_obj->length_signature);
+	signature_1 = OQS_MEM_malloc(lock_test_sig_obj->length_signature);
 
 	rc = OQS_SIG_STFL_sign(lock_test_sig_obj, signature_1, &signature_len_1, message_1, message_len_1, lock_test_sk);
 	OQS_TEST_CT_DECLASSIFY(&rc, sizeof rc);
@@ -866,7 +870,7 @@ static OQS_STATUS sig_stfl_test_sig_gen(const char *method_name) {
 	printf("Sig Gen 2 %s\n", method_name);
 	printf("================================================================================\n");
 
-	signature_2 = malloc(lock_test_sig_obj->length_signature);
+	signature_2 = OQS_MEM_malloc(lock_test_sig_obj->length_signature);
 
 	rc = OQS_SIG_STFL_sign(lock_test_sig_obj, signature_2, &signature_len_2, message_2, message_len_2, lock_test_sk);
 	OQS_TEST_CT_DECLASSIFY(&rc, sizeof rc);
@@ -927,7 +931,7 @@ static OQS_STATUS sig_stfl_test_secret_key_lock(const char *method_name, const c
 		goto err;
 	}
 
-	lock_test_public_key = malloc(lock_test_sig_obj->length_public_key * sizeof(uint8_t));
+	lock_test_public_key = OQS_MEM_malloc(lock_test_sig_obj->length_public_key * sizeof(uint8_t));
 
 	printf("================================================================================\n");
 	printf("Create stateful Secret Key  %s\n", method_name);
@@ -976,6 +980,8 @@ err:
 typedef struct thread_data {
 	const char *alg_name;
 	const char *katfile;
+	bool *bitflips_all;
+	size_t *bitflips;
 	OQS_STATUS rc;
 	// OQS_STATUS rc1;
 } thread_data_t;
@@ -990,6 +996,7 @@ void *test_query_key(void *arg) {
 	struct lock_test_data *td = arg;
 	printf("\n%s: Start Query Stateful Key info\n", __func__);
 	td->rc = sig_stfl_test_query_key(td->alg_name);
+	OQS_thread_stop();
 	printf("%s: End Query Stateful Key info\n\n", __func__);
 	return NULL;
 }
@@ -998,6 +1005,7 @@ void *test_sig_gen(void *arg) {
 	struct lock_test_data *td = arg;
 	printf("\n%s: Start Generate Stateful Signature\n", __func__);
 	td->rc = sig_stfl_test_sig_gen(td->alg_name);
+	OQS_thread_stop();
 	printf("%s: End Generate Stateful Signature\n\n", __func__);
 	return NULL;
 }
@@ -1006,19 +1014,22 @@ void *test_create_keys(void *arg) {
 	struct lock_test_data *td = arg;
 	printf("\n%s: Start Generate Keys\n", __func__);
 	td->rc = sig_stfl_test_secret_key_lock(td->alg_name, td->katfile);
+	OQS_thread_stop();
 	printf("%s: End Generate Stateful Keys\n\n", __func__);
 	return NULL;
 }
 
 void *test_correctness_wrapper(void *arg) {
 	struct thread_data *td = arg;
-	td->rc = sig_stfl_test_correctness(td->alg_name, td->katfile);
+	td->rc = sig_stfl_test_correctness(td->alg_name, td->katfile, td->bitflips_all, td->bitflips);
+	OQS_thread_stop();
 	return NULL;
 }
 
 void *test_secret_key_wrapper(void *arg) {
 	struct thread_data *td = arg;
 	td->rc = sig_stfl_test_secret_key(td->alg_name, td->katfile);
+	OQS_thread_stop();
 	return NULL;
 }
 #endif
@@ -1052,13 +1063,18 @@ static OQS_STATUS update_test_result( OQS_STATUS rc, int xmss_or_lms) {
 }
 
 int main(int argc, char **argv) {
+	OQS_STATUS  rc = OQS_ERROR, rc1 = OQS_ERROR;
 	OQS_init();
-	oqs_fstore_init();
+	rc = oqs_fstore_init();
+	if (rc != OQS_SUCCESS) {
+		OQS_destroy();
+		return EXIT_FAILURE;
+	}
 
 	printf("Testing stateful signature algorithms using liboqs version %s\n", OQS_version());
 
-	if (argc < 2) {
-		fprintf(stderr, "Usage: test_sig_stfl algname [katfile]\n");
+	if (argc < 2 || argc > 5) {
+		fprintf(stderr, "Usage: test_sig_stfl algname [katfile] [bitflips_msg] [bitflips_sig]\n");
 		fprintf(stderr, "  algname: ");
 		for (size_t i = 0; i < OQS_SIG_STFL_algs_length; i++) {
 			if (i > 0) {
@@ -1067,6 +1083,8 @@ int main(int argc, char **argv) {
 			fprintf(stderr, "%s", OQS_SIG_STFL_alg_identifier(i));
 		}
 		fprintf(stderr, "\n");
+		fprintf(stderr, " bitflips_msg: the number of random bitflips to perform for each EUF-CMA signature (\"all\" to flip every bit)\n");
+		fprintf(stderr, " bitflips_sig: the number of random bitflips to perform for each SUF-CMA signature (\"all\" to flip every bit)\n");
 		OQS_destroy();
 		return EXIT_FAILURE;
 	}
@@ -1085,8 +1103,26 @@ int main(int argc, char **argv) {
 		}
 	}
 
+	/* by default, flip 50 random bits of the message and signature (to test EUF-CMA and SUF-CMA, respectively) */
+	bool bitflips_all[2] = {false, false};
+	size_t bitflips[2] = {50, 50};
+	if (argc >= 4) {
+		if (strcmp(argv[2], "all") == 0) {
+			bitflips_all[0] = true;
+		} else {
+			bitflips[0] = (size_t)strtol(argv[3], NULL, 10);
+		}
+	}
+	if (argc == 5) {
+		if (strcmp(argv[3], "all") == 0) {
+			bitflips_all[1] = true;
+		} else {
+			bitflips[1] = (size_t)strtol(argv[4], NULL, 10);
+		}
+	}
+
 	/*
-	 * Tests executed by CI/DI only run algoritms that have been emabled.
+	 * Tests executed by CI/DI only run algoritms that have been enabled.
 	 *
 	 */
 	if (!OQS_SIG_STFL_alg_is_enabled(alg_name)) {
@@ -1110,10 +1146,14 @@ int main(int argc, char **argv) {
 #ifdef OQS_ENABLE_TEST_CONSTANT_TIME
 	OQS_randombytes_custom_algorithm(&TEST_SIG_STFL_randombytes);
 #else
-	OQS_randombytes_switch_algorithm("system");
+	rc = OQS_randombytes_switch_algorithm("system");
+	if (rc != OQS_SUCCESS) {
+		printf("Could not generate random data with system RNG\n");
+		OQS_destroy();
+		return EXIT_FAILURE;
+	}
 #endif
 
-	OQS_STATUS  rc = OQS_ERROR, rc1 = OQS_ERROR;
 	int exit_status = EXIT_SUCCESS;
 
 #if OQS_USE_PTHREADS_IN_TESTS
@@ -1125,18 +1165,18 @@ int main(int argc, char **argv) {
 	pthread_t sign_key_thread;
 	pthread_t query_key_thread;
 
-	thread_data_t td = {.alg_name = alg_name, .katfile = katfile, .rc = OQS_ERROR};
-	thread_data_t td_2 = {.alg_name = alg_name, .katfile = katfile, .rc = OQS_ERROR};
+	thread_data_t td = {.alg_name = alg_name, .katfile = katfile, .bitflips_all = bitflips_all, .bitflips = bitflips, .rc = OQS_ERROR};
+	thread_data_t td_2 = {.alg_name = alg_name, .katfile = katfile, .bitflips_all = bitflips_all, .bitflips = bitflips, .rc = OQS_ERROR};
 
 	lock_test_data_t td_create = {.alg_name = alg_name, .katfile = katfile, .rc = OQS_ERROR};
 	lock_test_data_t td_sign = {.alg_name = alg_name, .katfile = katfile, .rc = OQS_ERROR};
 	lock_test_data_t td_query = {.alg_name = alg_name, .katfile = katfile, .rc = OQS_ERROR};
 
-	test_sk_lock = (pthread_mutex_t *)malloc(sizeof(pthread_mutex_t));
+	test_sk_lock = (pthread_mutex_t *)OQS_MEM_malloc(sizeof(pthread_mutex_t));
 	if (test_sk_lock == NULL) {
 		goto err;
 	}
-	sk_lock = (pthread_mutex_t *)malloc(sizeof(pthread_mutex_t));
+	sk_lock = (pthread_mutex_t *)OQS_MEM_malloc(sizeof(pthread_mutex_t));
 	if (sk_lock == NULL) {
 		goto err;
 	}
@@ -1219,7 +1259,7 @@ err:
 #endif
 	return exit_status;
 #else
-	rc = sig_stfl_test_correctness(alg_name, katfile);
+	rc = sig_stfl_test_correctness(alg_name, katfile, bitflips_all, bitflips);
 	rc1 = sig_stfl_test_secret_key(alg_name, katfile);
 
 	OQS_destroy();
